@@ -1,18 +1,15 @@
 """
 The temporal fusion transformer is a powerful predictive model for forecasting timeseries
 """
-from typing import Callable, Union, List, Dict, Tuple, Any
+from typing import Callable, Union, List, Dict, Tuple
 
 import numpy as np
-from pytorch_lightning.metrics.metric import TensorMetric
 import torch
 from matplotlib import pyplot as plt
-from pytorch_ranger import Ranger
 from torch import nn
 from torch.nn.utils import rnn
-from torch.utils import data
 
-from pytorch_forecasting.models import BaseModel
+from pytorch_forecasting.models.base_model import BaseModel, CovariatesMixin
 from pytorch_forecasting.data import TimeSeriesDataSet
 from pytorch_forecasting.metrics import MultiHorizonMetric, QuantileLoss, SMAPE, MAE, RMSE, MAPE
 from pytorch_forecasting.models.temporal_fusion_transformer.sub_modules import (
@@ -27,7 +24,7 @@ from pytorch_forecasting.models.temporal_fusion_transformer.sub_modules import (
 from pytorch_forecasting.utils import autocorrelation, integer_histogram, get_embedding_size
 
 
-class TemporalFusionTransformer(BaseModel):
+class TemporalFusionTransformer(BaseModel, CovariatesMixin):
     def __init__(
         self,
         hidden_size: int = 16,
@@ -51,7 +48,6 @@ class TemporalFusionTransformer(BaseModel):
         embedding_sizes: Dict[str, Tuple[int, int]] = {},
         embedding_paddings: List[str] = [],
         embedding_labels: Dict[str, np.ndarray] = {},
-        real_scales: Dict[str, Tuple[float, float]] = {},
         learning_rate: float = 1e-3,
         log_interval: Union[int, float] = -1,
         log_val_interval: Union[int, float] = None,
@@ -72,7 +68,7 @@ class TemporalFusionTransformer(BaseModel):
             output_size: number of outputs (e.g. number of quantiles for QuantileLoss)
             loss: loss function taking prediction and targets
             attention_head_size: number of attention heads (4 is a good default)
-            max_encoder_length: length to encode
+            max_encoder_length: length to encode (can be far longer than the decoder length but does not have to be)
             static_categoricals: integer of positions of static categorical variables
             static_reals: integer of positions of static continuous variables
             time_varying_categoricals_encoder: integer of positions of categorical variables for encoder
@@ -89,9 +85,6 @@ class TemporalFusionTransformer(BaseModel):
                 embedding size
             embedding_paddings: list of indices for embeddings which transform the zero's embedding to a zero vector
             embedding_labels: dictionary mapping (string) indices to list of categorical labels
-            real_labels: dictionary mapping (string) indices to continuous variable names
-            real_scales: dictionary of tuples with mean and scale for each continuous variable that can be used to
-                transform them into their original shape
             learning_rate: learning rate
             log_interval: log predictions every x batches, do not log if 0 or less, log interpretation if > 0. If < 1.0
                 , will log multiple entries per batch. Defaults to -1.
@@ -133,7 +126,9 @@ class TemporalFusionTransformer(BaseModel):
                 else:
                     padding_idx = None
                 self.input_embeddings[name] = nn.Embedding(
-                    self.hparams.embedding_sizes[name][0], embedding_size, padding_idx=padding_idx,
+                    self.hparams.embedding_sizes[name][0],
+                    embedding_size,
+                    padding_idx=padding_idx,
                 )
 
         # continuous variable processing
@@ -306,13 +301,6 @@ class TemporalFusionTransformer(BaseModel):
 
         self.output_layer = nn.Linear(self.hparams.hidden_size, self.hparams.output_size)
 
-    @property
-    def categorical_groups_mapping(self) -> Dict[str, str]:
-        groups = {}
-        for group_name, sublist in self.categorical_groups.items():
-            groups.update({name: group_name for name in sublist})
-        return groups
-
     @property
     def static_variables(self) -> List[str]:
         return self.hparams.static_categoricals + self.hparams.static_reals
@@ -327,7 +315,10 @@ class TemporalFusionTransformer(BaseModel):
 
     @classmethod
     def from_dataset(
-        cls, dataset: TimeSeriesDataSet, allowed_encoder_known_variable_names: List[str] = None, **kwargs,
+        cls,
+        dataset: TimeSeriesDataSet,
+        allowed_encoder_known_variable_names: List[str] = None,
+        **kwargs,
     ):
         """
         Create model from dataset.
@@ -471,7 +462,8 @@ class TemporalFusionTransformer(BaseModel):
             for name in self.hparams.time_varying_categoricals_encoder + self.hparams.time_varying_reals_encoder
         }
         embeddings_varying_encoder, encoder_sparse_weights = self.encoder_variable_selection(
-            embeddings_varying_encoder, static_context_variable_selection[:, :max_encoder_length],
+            embeddings_varying_encoder,
+            static_context_variable_selection[:, :max_encoder_length],
         )
 
         embeddings_varying_decoder = {
@@ -479,7 +471,8 @@ class TemporalFusionTransformer(BaseModel):
             for name in self.hparams.time_varying_categoricals_decoder + self.hparams.time_varying_reals_decoder
         }
         embeddings_varying_decoder, decoder_sparse_weights = self.decoder_variable_selection(
-            embeddings_varying_decoder, static_context_variable_selection[:, max_encoder_length:],
+            embeddings_varying_decoder,
+            static_context_variable_selection[:, max_encoder_length:],
         )
 
         # LSTM
@@ -515,7 +508,10 @@ class TemporalFusionTransformer(BaseModel):
         decoder_output, _ = rnn.pad_packed_sequence(decoder_output, batch_first=True)
 
         # run local decoder
-        decoder_output, _ = self.lstm_decoder(embeddings_varying_decoder, (hidden, cell),)
+        decoder_output, _ = self.lstm_decoder(
+            embeddings_varying_decoder,
+            (hidden, cell),
+        )
 
         # skip connection over lstm
         lstm_output_encoder = self.post_lstm_gate_encoder(encoder_output)
